@@ -36,7 +36,7 @@ from psyneulink.core.globals.keywords import \
     NORMED_L0_SIMILARITY, OBJECTIVE_FUNCTION_TYPE, SIZE, STABILITY_FUNCTION
 from psyneulink.core.globals.parameters import Parameter
 from psyneulink.core.globals.preferences.basepreferenceset import is_pref_set
-from psyneulink.core.globals.utilities import is_distance_metric, safe_len
+from psyneulink.core.globals.utilities import is_distance_metric, safe_len, convert_to_np_array
 from psyneulink.core.globals.utilities import is_iterable
 
 
@@ -322,6 +322,9 @@ class Stability(ObjectiveFunction):
                                  target_set=target_set,
                                  context=context)
 
+    def _parse_metric_fct_variable(self, variable):
+        return convert_to_np_array([variable, variable])
+
     def _instantiate_attributes_before_function(self, function=None, context=None):
         """Instantiate matrix
 
