@@ -431,7 +431,7 @@ FEATURES = 'features'
 
 
 def _parse_feature_values_from_variable(variable):
-    return np.array(np.array(variable[1:]).tolist())
+    return convert_to_np_array(np.array(variable[1:]).tolist())
 
 
 class OptimizationControlMechanismError(Exception):
@@ -740,7 +740,7 @@ class OptimizationControlMechanism(ControlMechanism):
         if agent_rep is None:
             if context.source==ContextFlags.COMMAND_LINE:
                 # Temporarily name InputPort
-                self._assign_deferred_init_name(self.__class__.__name__, context)
+                self._assign_deferred_init_name(self.__class__.__name__)
                 # Store args for deferred initialization
                 self._store_deferred_init_args(**locals())
 
@@ -794,13 +794,15 @@ class OptimizationControlMechanism(ControlMechanism):
 
         # If any features were specified (assigned to self.input_ports in __init__):
         if self.input_ports:
-            self.input_ports = _parse_shadow_inputs(self, self.input_ports)
-            self.input_ports = self._parse_feature_specs(self.input_ports, self.feature_function)
+            input_ports = _parse_shadow_inputs(self, self.input_ports)
+            input_ports = self._parse_feature_specs(input_ports, self.feature_function)
             # Insert primary InputPort for outcome from ObjectiveMechanism;
             #     assumes this will be a single scalar value and must be named OUTCOME by convention of ControlSignal
-            self.input_ports.insert(0, outcome_input_port),
+            input_ports.insert(0, outcome_input_port),
         else:
-            self.input_ports = [outcome_input_port]
+            input_ports = [outcome_input_port]
+
+        self.parameters.input_ports._set(input_ports, context)
 
         # Configure default_variable to comport with full set of input_ports
         self.defaults.variable, _ = self._handle_arg_input_ports(self.input_ports)
@@ -877,7 +879,7 @@ class OptimizationControlMechanism(ControlMechanism):
             self.parameters.search_space._set(corrected_search_space, context)
 
         # Assign parameters to function (OptimizationFunction) that rely on OptimizationControlMechanism
-        self.function.reset({
+        self.function.reset(**{
             DEFAULT_VARIABLE: self.parameters.control_allocation._get(context),
             OBJECTIVE_FUNCTION: self.evaluation_function,
             # SEARCH_FUNCTION: self.search_function,
@@ -1300,7 +1302,8 @@ class OptimizationControlMechanism(ControlMechanism):
     # FIX: SHOULD BE GENERALIZED AS SOMETHING LIKE update_feature_values
 
     @tc.typecheck
-    def add_features(self, features):
+    @handle_external_context()
+    def add_features(self, features, context=None):
         """Add InputPorts and Projections to OptimizationControlMechanism for features used to
         predict `net_outcome <ControlMechanism.net_outcome>`
 
@@ -1309,7 +1312,7 @@ class OptimizationControlMechanism(ControlMechanism):
 
         if features:
             features = self._parse_feature_specs(features=features,
-                                                 context=Context(source=ContextFlags.COMMAND_LINE))
+                                                 context=context)
         self.add_ports(InputPort, features)
 
     @tc.typecheck
