@@ -375,7 +375,7 @@ from psyneulink.core.components.mechanisms.processing.processingmechanism import
 from psyneulink.core.components.ports.outputport import OutputPort
 from psyneulink.core.components.ports.inputport import InputPort, INPUT_PORT
 from psyneulink.core.components.ports.port import _parse_port_spec
-from psyneulink.core.globals.context import ContextFlags
+from psyneulink.core.globals.context import ContextFlags, handle_external_context
 from psyneulink.core.globals.keywords import \
     CONTROL, EXPONENT, EXPONENTS, FUNCTION, LEARNING, MATRIX, NAME, OBJECTIVE_MECHANISM, OUTCOME, OWNER_VALUE, \
     PARAMS, PREFERENCE_SET_NAME, PROJECTION, PROJECTIONS, PORT_TYPE, VARIABLE, WEIGHT, WEIGHTS
@@ -666,6 +666,7 @@ class ObjectiveMechanism(ProcessingMechanism_Base):
             port.name = "Value of {} [{}]".format(proj.sender.owner.name, proj.sender.name)
             register_instance(port, port.name, InputPort, self._portRegistry, INPUT_PORT)
 
+    @handle_external_context(source=ContextFlags.METHOD)
     def add_to_monitor(self, monitor_specs, context=None):
         """Instantiate `OutputPorts <OutputPort>` to be monitored by the ObjectiveMechanism.
 
@@ -730,11 +731,6 @@ class ObjectiveMechanism(ProcessingMechanism_Base):
             else:
                 reference_value.append(projection_tuple.port.value)
 
-        if not context:
-            from psyneulink.core.globals.context import Context
-            context = Context()
-        context.source = ContextFlags.METHOD
-
         input_ports = self._instantiate_input_ports(monitor_specs=monitor_specs,
                                                       reference_value=reference_value,
                                                       context=context
