@@ -946,7 +946,7 @@ class OutputPort(Port_Base):
         # if owner is None or reference_value is None:
         if owner is None:
             # Temporarily name OutputPort
-            self._assign_deferred_init_name(name, context)
+            self._assign_deferred_init_name(name)
             # Store args for deferred initialization
             self._store_deferred_init_args(**locals())
 
@@ -1500,7 +1500,7 @@ def _instantiate_output_ports(owner, output_ports=None, context=None):
     if context.source & (ContextFlags.COMMAND_LINE | ContextFlags.METHOD):
         owner.output_ports.extend(port_list)
     else:
-        owner.output_ports = port_list
+        owner.parameters.output_ports._set(port_list, context)
 
     # Assign value of require_projection_in_composition
     for port in owner.output_ports:
