@@ -274,6 +274,7 @@ import collections
 import copy
 import itertools
 import logging
+import toposort
 import types
 import typing
 import weakref
@@ -284,7 +285,7 @@ from psyneulink.core.rpc.graph_pb2 import Entry, ndArray
 from psyneulink.core.globals.context import Context, ContextError, ContextFlags, _get_time, handle_external_context
 from psyneulink.core.globals.context import time as time_object
 from psyneulink.core.globals.log import LogCondition, LogEntry, LogError
-from psyneulink.core.globals.utilities import call_with_pruned_args, copy_iterable_with_shared, get_alias_property_getter, get_alias_property_setter, get_deepcopy_with_shared, unproxy_weakproxy
+from psyneulink.core.globals.utilities import call_with_pruned_args, copy_iterable_with_shared, get_alias_property_getter, get_alias_property_setter, get_deepcopy_with_shared, unproxy_weakproxy, create_union_set
 
 __all__ = [
     'Defaults', 'get_validator_by_function', 'Parameter', 'ParameterAlias', 'ParameterError',
@@ -490,6 +491,29 @@ class ParametersTemplate:
         except TypeError:
             self._owner_ref = value
 
+    @property
+    def _in_dependency_order(self):
+        """
+            Returns:
+                list[Parameter] - a list of Parameters such that any
+                Parameter is placed before all of its
+                `dependencies <Parameter.dependencies>`
+        """
+        parameter_function_ordering = list(toposort.toposort({
+            p.name: p.dependencies for p in self if p.dependencies is not None
+        }))
+        parameter_function_ordering = list(
+            itertools.chain.from_iterable(parameter_function_ordering)
+        )
+
+        def ordering(p):
+            try:
+                return parameter_function_ordering.index(p.name)
+            except ValueError:
+                return -1
+
+        return sorted(self, key=ordering)
+
 
 class Defaults(ParametersTemplate):
     """
@@ -730,6 +754,19 @@ class Parameter(ParameterBase):
             _instantiate_parameter_classes or validated for variable
             shape
 
+        dependencies
+            if not None, this contains a set of Parameter
+            names corresponding to Parameters whose values must be
+            instantiated before that of this Parameter
+
+            :default: None
+
+        initializer
+            the name of another Parameter that serves as this
+            Parameter's `initializer <StatefulFunction.initializers>`
+
+            :default: None
+
     """
     # The values of these attributes will never be inherited from parent Parameters
     # KDM 7/12/18: consider inheriting ONLY default_value?
@@ -739,7 +776,7 @@ class Parameter(ParameterBase):
     # display if the function is True based on the value of the attribute
     _hidden_if_unset_attrs = {
         'aliases', 'getter', 'setter', 'constructor_argument', 'spec',
-        'modulation_combination_function', 'valid_types'
+        'modulation_combination_function', 'valid_types', 'initializer'
     }
     _hidden_if_false_attrs = {'read_only', 'modulable', 'fallback_default', 'retain_old_simulation_data'}
     _hidden_when = {
@@ -791,6 +828,8 @@ class Parameter(ParameterBase):
         parse_spec=False,
         valid_types=None,
         reference=False,
+        dependencies=None,
+        initializer=None,
         _owner=None,
         _inherited=False,
         # this stores a reference to the Parameter object that is the
@@ -818,6 +857,9 @@ class Parameter(ParameterBase):
             else:
                 valid_types = (valid_types, )
 
+        if dependencies is not None:
+            dependencies = create_union_set(dependencies)
+
         super().__init__(
             default_value=default_value,
             name=name,
@@ -847,6 +889,8 @@ class Parameter(ParameterBase):
             parse_spec=parse_spec,
             valid_types=valid_types,
             reference=reference,
+            dependencies=dependencies,
+            initializer=initializer,
             _inherited=_inherited,
             _inherited_source=_inherited_source,
             _user_specified=_user_specified,
@@ -1215,12 +1259,37 @@ class Parameter(ParameterBase):
                 kwargs
                     any additional arguments to be passed to this `Parameter`'s `setter` if it exists
         """
+        from psyneulink.core.components.component import Component
+
         if not override and self.read_only:
             raise ParameterError('Parameter \'{0}\' is read-only. Set at your own risk. Pass override=True to force set.'.format(self.name))
 
-        return self._set(self._parse(value), context, skip_history, skip_log, **kwargs)
+        value = self._set(self._parse(value), context, skip_history, skip_log, **kwargs)
 
-    def _set(self, value, context=None, skip_history=False, skip_log=False, **kwargs):
+        try:
+            value = value.__self__
+        except AttributeError:
+            pass
+
+        if isinstance(value, Component):
+            owner = self._owner._owner
+            if value not in owner._parameter_components:
+                if not owner.is_initializing:
+                    value._initialize_from_context(context)
+                    owner._parameter_components.add(value)
+
+                    try:
+                        value._update_default_variable(owner._get_parsed_variable(self, context=context), context)
+                    except TypeError as e:
+                        if (
+                            f'unsupported for {value.__class__.__name__}' not in str(e)
+                            and f'unsupported for {owner.__class__.__name__}' not in str(e)
+                        ):
+                            raise
+
+        return value
+
+    def _set(self, value, context, skip_history=False, skip_log=False, **kwargs):
         if not self.stateful:
             execution_id = None
         else:
