@@ -132,7 +132,8 @@ class BasePreferenceSet(PreferenceSet):
         Implement the following preferences:
             - verbose (bool): enables/disables reporting of (non-exception) warnings and system function
             - paramValidation (bool):  enables/disables run-time validation of the execute method of a Function object
-            - reportOutput (bool): enables/disables reporting of execution of execute method
+            - reportOutput ([bool, str]): enables/disables reporting of execution of execute method:
+              True prints input/output, 'params' or 'parameters' includes parameter values
             - log (bool): sets LogCondition for a given Component
             - functionRunTimeParams (Modulation): uses run-time params to modulate execute method params
         Implement the following preference levels:
