@@ -132,6 +132,7 @@ __all__ = [
     'scalar_distance', 'sinusoid',
     'tensor_power', 'TEST_CONDTION', 'type_match',
     'underscore_to_camelCase', 'UtilitiesError', 'unproxy_weakproxy', 'create_union_set', 'merge_dictionaries',
+    'contains_type'
 ]
 
 logger = logging.getLogger(__name__)
@@ -713,7 +714,10 @@ def get_deepcopy_with_shared(shared_keys=frozenset(), shared_types=()):
             if k in shared_keys or isinstance(v, shared_types):
                 res_val = v
             else:
-                res_val = copy.deepcopy(v, memo)
+                try:
+                    res_val = copy_iterable_with_shared(v, shared_types, memo)
+                except TypeError:
+                    res_val = copy.deepcopy(v, memo)
             setattr(result, k, res_val)
         return result
 
@@ -738,7 +742,7 @@ def copy_iterable_with_shared(obj, shared_types=None, memo=None):
             new_k = k if isinstance(k, shared_types) else copy.deepcopy(k, memo)
 
             if isinstance(v, all_types_using_recursion):
-                new_v = copy_iterable_with_shared(v, shared_types)
+                new_v = copy_iterable_with_shared(v, shared_types, memo)
             elif isinstance(v, shared_types):
                 new_v = v
             else:
@@ -766,7 +770,7 @@ def copy_iterable_with_shared(obj, shared_types=None, memo=None):
 
         for item in obj:
             if isinstance(item, all_types_using_recursion):
-                new_item = copy_iterable_with_shared(item, shared_types)
+                new_item = copy_iterable_with_shared(item, shared_types, memo)
             elif isinstance(item, shared_types):
                 new_item = item
             else:
@@ -1141,6 +1145,13 @@ class ContentAddressableList(UserList):
     #                                                                     repr(self[i].value))
     #                                              for i in range(len(self))]))
 
+    def __copy__(self):
+        cls = self.__class__
+        result = cls.__new__(cls)
+        result.__dict__.update(self.__dict__)
+        result.data = self.data.copy()
+        return result
+
     def __getitem__(self, key):
         if key is None:
             raise KeyError("None is not a legal key for {}".format(self.name))
@@ -1779,3 +1790,25 @@ def gen_friendly_comma_str(items):
             divider = f',{divider}'
 
         return f"{', '.join(items[:-1])}{divider}{items[-1]}"
+
+
+def contains_type(
+    arr: collections.abc.Iterable,
+    typ: typing.Union[type, typing.Tuple[type, ...]]
+) -> bool:
+    """
+        Returns:
+            True if **arr** is a possibly nested Iterable that contains
+            an instance of **typ** (or one type in **typ** if tuple)
+
+        Note: `isinstance(**arr**, **typ**)` should be used to check
+        **arr** itself if needed
+    """
+    try:
+        for a in arr:
+            if isinstance(a, typ) or (a is not arr and contains_type(a, typ)):
+                return True
+    except TypeError:
+        pass
+
+    return False
