@@ -10,6 +10,7 @@
 import ast
 import warnings
 import numpy as np
+from functools import reduce
 
 from llvmlite import ir
 from contextlib import contextmanager
@@ -28,16 +29,28 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
         self.func_params = func_params
         self.arg_in = arg_in
         self.arg_out = arg_out
-        self.register = {}
 
         #setup default functions
-        def _vec_sum(x):
-            dim = len(x.type.pointee)
-            output_scalar = builder.alloca(ctx.float_ty)
-            # Get the pointer to the first element of the array to convert from [? x double]* -> double*
-            vec_u = builder.gep(x, [ctx.int32_ty(0), ctx.int32_ty(0)])
-            builder.call(ctx.import_llvm_function("__pnl_builtin_vec_sum"), [vec_u, ctx.int32_ty(dim), output_scalar])
-            return output_scalar
+        def _list_sum(x):
+            # HACK: Obtain polymorphic addition function by visiting the add node
+            # this should ideally be moved to an explicit helper
+            add_func = self.visit_Add(None)
+
+            total_sum = builder.alloca(x.type.pointee.element)
+            builder.store(total_sum.type.pointee(None), total_sum)
+            with helpers.array_ptr_loop(builder, x, "list_sum") as (b, idx):
+                curr_val = b.gep(x, [ctx.int32_ty(0), idx])
+                tmp = add_func(total_sum, curr_val)
+                if helpers.is_pointer(tmp):
+                    tmp = builder.load(tmp)
+                b.store(tmp, total_sum)
+            return total_sum
+
+        def _len(x):
+            x_ty = x.type
+            if helpers.is_pointer(x):
+                x_ty = x_ty.pointee
+            return ctx.float_ty(len(x_ty))
 
         def _tanh(x):
             output_ptr = builder.alloca(x.type.pointee)
@@ -49,12 +62,60 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
             helpers.call_elementwise_operation(self.ctx, self.builder, x, helpers.exp, output_ptr)
             return output_ptr
 
-        self.register['sum'] = _vec_sum
+        # numpy's max function differs greatly from that of python's buiiltin max
+        # see: https://numpy.org/doc/stable/reference/generated/numpy.amax.html#numpy.amax
+        def _max_numpy(x):
+            assert helpers.is_vector(x) or helpers.is_2d_matrix(x), "Attempted to call max on invalid variable! Only 1-d and 2-d lists are supported!"
+            curr = builder.alloca(ctx.float_ty)
+            builder.store(ctx.float_ty('NaN'), curr)
+            for (element_ptr,) in helpers.recursive_iterate_arrays(ctx, builder, x):
+                element = builder.load(element_ptr)
+                greater = builder.fcmp_unordered('>', element, builder.load(curr))
+                with builder.if_then(greater):
+                    builder.store(element, curr)
+            return curr
+
+        # see: https://docs.python.org/3/library/functions.html#max
+        def _max(*args):
+            if len(args) == 1 and helpers.is_vector(args[0]):
+                curr = builder.alloca(ctx.float_ty)
+                builder.store(ctx.float_ty('NaN'), curr)
+                for (element_ptr,) in helpers.recursive_iterate_arrays(ctx, builder, args[0]):
+                    element = builder.load(element_ptr)
+                    greater = builder.fcmp_unordered('>', element, builder.load(curr))
+                    with builder.if_then(greater):
+                        builder.store(element, curr)
+                return curr
+            elif len(args) > 1 and all(a.type == args[0].type for a in args):
+                curr = builder.alloca(ctx.float_ty)
+                builder.store(ctx.float_ty('NaN'), curr)
+                for element in args:
+                    if helpers.is_pointer(element):
+                        element = builder.load(element)
+                    greater = builder.fcmp_unordered('>', element, builder.load(curr))
+                    with builder.if_then(greater):
+                        builder.store(element, curr)
+                return curr
+            assert False, "Attempted to call max with invalid arguments!"
+        self.register = {
+            "sum": _list_sum,
+            "len": _len,
+            "float": ctx.float_ty,
+            "int": ctx.int32_ty,
+            "max": _max,
+        }
 
         # setup numpy
         numpy_handlers = {
             'tanh': _tanh,
-            'exp': _exp
+            'exp': _exp,
+            'equal': self._generate_fcmp_handler(self.ctx, self.builder, "=="),
+            'not_equal': self._generate_fcmp_handler(self.ctx, self.builder, "!="),
+            'less': self._generate_fcmp_handler(self.ctx, self.builder, "<"),
+            'less_equal': self._generate_fcmp_handler(self.ctx, self.builder, "<="),
+            'greater': self._generate_fcmp_handler(self.ctx, self.builder, ">"),
+            'greater_equal': self._generate_fcmp_handler(self.ctx, self.builder, ">="),
+            "max": _max_numpy,
         }
 
         for k, v in func_globals.items():
@@ -62,8 +123,8 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
                 self.register[k] = numpy_handlers
 
         name_constants = {
-            True: ir.IntType(1)(1),
-            False: ir.IntType(1)(0),
+            True: ctx.bool_ty(1),
+            False: ctx.bool_ty(0),
         }
         self.name_constants = name_constants
         super().__init__()
@@ -228,12 +289,62 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
             elif helpers.is_floating_point(x) and helpers.is_2d_matrix(y):
                 return self._generate_binop(y, x, _mul_mat_scalar)
             elif helpers.is_vector(x) and helpers.is_vector(y):
+                if x.type != y.type:
+                    # Special case: Cast y into scalar if it can be done
+                    if helpers.get_array_shape(y) == [1]:
+                        y = self.builder.gep(y, [self.ctx.int32_ty(0), self.ctx.int32_ty(0)])
+                        return self._generate_binop(x, y, _mul_vec_scalar)
                 return self._generate_binop(x, y, _mul_vec)
             elif helpers.is_2d_matrix(x) and helpers.is_2d_matrix(y):
                 return self._generate_binop(x, y, _mul_mat)
 
         return _mul
 
+    def visit_Div(self, node):
+        def _div_array(ctx, builder, u, v):
+            assert u.type == v.type
+            output_ptr = builder.alloca(u.type.pointee)
+
+            for (u_ptr, v_ptr, out_ptr) in helpers.recursive_iterate_arrays(ctx, builder, u, v, output_ptr):
+                u_val = builder.load(u_ptr)
+                v_val = builder.load(v_ptr)
+                builder.store(builder.fdiv(u_val, v_val), out_ptr)
+
+            return output_ptr
+
+        def _div_array_scalar(ctx, builder, array, s):
+            output_ptr = builder.alloca(array.type.pointee)
+            helpers.call_elementwise_operation(ctx, builder, array, lambda ctx, builder, x:  builder.fdiv(x, s), output_ptr)
+
+            return output_ptr
+
+        def _div_scalar_array(ctx, builder, s, array):
+            output_ptr = builder.alloca(array.type.pointee)
+            helpers.call_elementwise_operation(ctx, builder, array, lambda ctx, builder, x:  builder.fdiv(s, x), output_ptr)
+
+            return output_ptr
+
+        def _div(x, y):
+            if helpers.is_floating_point(x) and helpers.is_floating_point(y):
+                return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fdiv(x, y))
+            elif helpers.is_floating_point(x) and (helpers.is_2d_matrix(y) or helpers.is_vector(y)):
+                return self._generate_binop(x, y, _div_scalar_array)
+            elif helpers.is_vector(x) and helpers.is_floating_point(y):
+                return self._generate_binop(x, y, _div_array_scalar)
+            elif helpers.is_2d_matrix(x) and helpers.is_floating_point(y):
+                return self._generate_binop(x, y, _div_array_scalar)
+            elif helpers.is_vector(x) and helpers.is_vector(y):
+                if x.type != y.type:
+                    # Special case: Cast y into scalar if it can be done
+                    if helpers.get_array_shape(y) == [1]:
+                        y = self.builder.gep(y, [self.ctx.int32_ty(0), self.ctx.int32_ty(0)])
+                        return self._generate_binop(x, y, _div_array_scalar)
+                return self._generate_binop(x, y, _div_array)
+            elif helpers.is_2d_matrix(x) and helpers.is_2d_matrix(y):
+                return self._generate_binop(x, y, _div_array)
+            assert False, f"Unable to divide arguments {x}, {y}"
+        return _div
+
     def _generate_unop(self, x, callback):
         if helpers.is_floating_point(x) and helpers.is_pointer(x):
             x = self.builder.load(x)
@@ -260,6 +371,50 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
 
     def visit_Attribute(self, node):
         val = self.visit(node.value)
+
+        # special case numpy attributes
+        if node.attr == "shape":
+            shape = helpers.get_array_shape(val)
+            return ir.ArrayType(self.ctx.float_ty, len(shape))(shape)
+        elif node.attr == "flatten":
+            def flatten():
+                shape = helpers.get_array_shape(val)
+                flattened_size = reduce(lambda x, y: x * y, shape)
+                flattened_ty = ir.ArrayType(self.ctx.float_ty, flattened_size)
+                flattened_array = self.builder.alloca(flattened_ty)
+                index_var = self.builder.alloca(self.ctx.int32_ty, name="flattened_index_var_loc")
+                self.builder.store(self.ctx.int32_ty(0), index_var)
+                for (array_ptr,) in helpers.recursive_iterate_arrays(self.ctx, self.builder, val):
+                    index = self.builder.load(index_var, name="flattened_index_var")
+                    flattened_array_ptr = self.builder.gep(flattened_array, [self.ctx.int32_ty(0), index])
+                    array_val = self.builder.load(array_ptr)
+                    self.builder.store(array_val, flattened_array_ptr)
+                    index = self.builder.add(index, self.ctx.int32_ty(1), name="flattened_index_var_inc")
+                    self.builder.store(index, index_var)
+                return flattened_array
+            return flatten
+        elif node.attr == "astype":
+            def astype(ty):
+                def _convert(ctx, builder, x):
+                    if helpers.is_pointer(x):
+                        x = builder.load(x)
+                    if helpers.is_integer(x) and ty is ctx.float_ty:
+                        if helpers.is_boolean(x):
+                            return builder.uitofp(x, ty)
+                        return builder.sitofp(x, ty)
+                    elif helpers.is_floating_point(x) and ty is self.register["int"]:
+                        return builder.fptosi(x, ty)
+                    elif (helpers.is_floating_point(x) and ty is ctx.float_ty):
+                        return x
+                if helpers.is_scalar(val):
+                    return _convert(self.ctx, self.builder, val)
+                else:
+                    output_ptr = self.builder.alloca(helpers.array_from_shape(helpers.get_array_shape(val), ty))
+                    helpers.call_elementwise_operation(self.ctx, self.builder, val, _convert, output_ptr)
+                    return output_ptr
+            # we only support float types
+            return astype
+
         return val[node.attr]
 
     def visit_Num(self, node):
@@ -273,8 +428,6 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
                 self.register[id] = value
             else:
                 to_store = value
-                if helpers.is_pointer(value):
-                    to_store = builder.load(value)
 
                 target = self.visit(target)
                 self.builder.store(to_store, target)
@@ -293,7 +446,10 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
         elements = [self.builder.load(element) if helpers.is_pointer(element) else element for element in elements]
 
         element_types = [element.type for element in elements]
-        ret_list = self.builder.alloca(ir.LiteralStructType(element_types))
+        if len(element_types) > 0 and all(x == element_types[0] for x in element_types):
+            ret_list = self.builder.alloca(ir.ArrayType(element_types[0], len(element_types)))
+        else:
+            ret_list = self.builder.alloca(ir.LiteralStructType(element_types))
 
         for idx, element in enumerate(elements):
             self.builder.store(element, self.builder.gep(ret_list, [self.ctx.int32_ty(0), self.ctx.int32_ty(idx)]))
@@ -348,53 +504,68 @@ class UserDefinedFunctionVisitor(ast.NodeVisitor):
 
         return _or
 
-    def visit_Eq(self, node):
-        def _eq(x, y):
-            assert helpers.is_floating_point(x), f"{x} is not a floating point type!"
-            assert helpers.is_floating_point(y), f"{y} is not a floating point type!"
-            return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fcmp_ordered('==', x, y))
+    def _generate_fcmp_handler(self, ctx, builder, cmp):
+        def _cmp_array(ctx, builder, u, v):
+            assert u.type == v.type
+            shape = helpers.get_array_shape(u)
+            output_ptr = builder.alloca(helpers.array_from_shape(shape, ctx.bool_ty))
 
-        return _eq
+            for (u_ptr, v_ptr, out_ptr) in helpers.recursive_iterate_arrays(ctx, builder, u, v, output_ptr):
+                u_val = builder.load(u_ptr)
+                v_val = builder.load(v_ptr)
+                builder.store(builder.fcmp_ordered(cmp, u_val, v_val), out_ptr)
+
+            return output_ptr
+
+        def _cmp_array_scalar(ctx, builder, array, s):
+            shape = helpers.get_array_shape(array)
+            output_ptr = builder.alloca(helpers.array_from_shape(shape, ctx.bool_ty))
+            helpers.call_elementwise_operation(ctx, builder, array, lambda ctx, builder, x:  builder.fcmp_ordered(cmp, x, s), output_ptr)
+
+            return output_ptr
+
+        def _cmp_scalar_array(ctx, builder, s, array):
+            shape = helpers.get_array_shape(array)
+            output_ptr = builder.alloca(helpers.array_from_shape(shape, ctx.bool_ty))
+            helpers.call_elementwise_operation(ctx, builder, array, lambda ctx, builder, x:  builder.fcmp_ordered(cmp, s, x), output_ptr)
+
+            return output_ptr
+
+        def _cmp(x, y):
+            if helpers.is_floating_point(x) and helpers.is_floating_point(y):
+                return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fcmp_ordered(cmp, x, y))
+            elif helpers.is_vector(x) and helpers.is_floating_point(y):
+                return self._generate_binop(x, y, _cmp_array_scalar)
+            elif helpers.is_floating_point(x) and helpers.is_vector(y):
+                return self._generate_binop(x, y, _cmp_scalar_array)
+            elif helpers.is_2d_matrix(x) and helpers.is_floating_point(y):
+                return self._generate_binop(x, y, _cmp_array_scalar)
+            elif helpers.is_floating_point(x) and helpers.is_2d_matrix(y):
+                return self._generate_binop(x, y, _cmp_scalar_array)
+            elif helpers.is_vector(x) and helpers.is_vector(y):
+                return self._generate_binop(x, y, _cmp_array)
+            elif helpers.is_2d_matrix(x) and helpers.is_2d_matrix(y):
+                return self._generate_binop(x, y, _cmp_array)
+
+        return _cmp
+
+    def visit_Eq(self, node):
+        return self._generate_fcmp_handler(self.ctx, self.builder, "==")
 
     def visit_NotEq(self, node):
-        def _neq(x, y):
-            assert helpers.is_floating_point(x), f"{x} is not a floating point type!"
-            assert helpers.is_floating_point(y), f"{y} is not a floating point type!"
-            return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fcmp_ordered('!=', x, y))
-
-        return _neq
+        return self._generate_fcmp_handler(self.ctx, self.builder, "!=")
 
     def visit_Lt(self, node):
-        def _lt(x, y):
-            assert helpers.is_floating_point(x), f"{x} is not a floating point type!"
-            assert helpers.is_floating_point(y), f"{y} is not a floating point type!"
-            return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fcmp_ordered('<', x, y))
-
-        return _lt
+        return self._generate_fcmp_handler(self.ctx, self.builder, "<")
 
     def visit_LtE(self, node):
-        def _lte(x, y):
-            assert helpers.is_floating_point(x), f"{x} is not a floating point type!"
-            assert helpers.is_floating_point(y), f"{y} is not a floating point type!"
-            return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fcmp_ordered('<=', x, y))
-
-        return _lte
+        return self._generate_fcmp_handler(self.ctx, self.builder, "<=")
 
     def visit_Gt(self, node):
-        def _gt(x, y):
-            assert helpers.is_floating_point(x), f"{x} is not a floating point type!"
-            assert helpers.is_floating_point(y), f"{y} is not a floating point type!"
-            return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fcmp_ordered('>', x, y))
-
-        return _gt
+        return self._generate_fcmp_handler(self.ctx, self.builder, ">")
 
     def visit_GtE(self, node):
-        def _gte(x, y):
-            assert helpers.is_floating_point(x), f"{x} is not a floating point type!"
-            assert helpers.is_floating_point(y), f"{y} is not a floating point type!"
-            return self._generate_binop(x, y, lambda ctx, builder, x, y: builder.fcmp_ordered('>=', x, y))
-
-        return _gte
+        return self._generate_fcmp_handler(self.ctx, self.builder, ">=")
 
     def visit_Compare(self, node):
         comp_val = self.visit(node.left)
@@ -468,7 +639,7 @@ def gen_node_wrapper(ctx, composition, node, *, tags:frozenset):
         cond_ty = cond_gen.get_condition_struct_type().as_pointer()
         args.append(cond_ty)
 
-    builder = ctx.create_llvm_function(args, node, node_function.name, tags=tags,
+    builder = ctx.create_llvm_function(args, node, tags=tags,
                                        return_type=node_function.type.pointee.return_type)
     llvm_func = builder.function
     for a in llvm_func.args:
@@ -672,23 +843,32 @@ def gen_composition_exec(ctx, composition, *, tags:frozenset):
 
         nodes_states = helpers.get_param_ptr(builder, composition, state, "nodes")
 
+        # Allocate temporary output storage
+        output_storage = builder.alloca(data.type.pointee, name="output_storage")
+
+        # Get locations of number of executions.
         num_exec_locs = {}
         for idx, node in enumerate(composition._all_nodes):
-            #FIXME: This skips nested compositions
-            from psyneulink import Composition
-            if isinstance(node, Composition):
-                continue
             node_state = builder.gep(nodes_states, [ctx.int32_ty(0),
                                                     ctx.int32_ty(idx)])
             num_exec_locs[node] = helpers.get_state_ptr(builder, node,
                                                         node_state,
                                                         "num_executions")
 
-        # Reset internal TRIAL clock for each node
+        # Generate pointers to 'is_finished' callbacks
+        is_finished_callbacks = {}
+        for node in composition.nodes:
+            args = [state, params, comp_in, data, output_storage]
+            wrapper = ctx.get_node_wrapper(composition, node)
+            is_finished_callbacks[node] = (wrapper, args)
+
+
+        # Reset internal TRIAL/PASS/TIME_STEP clock for each node
         for time_loc in num_exec_locs.values():
-            num_exec_time_ptr = builder.gep(time_loc, [ctx.int32_ty(0),
-                                                       ctx.int32_ty(TimeScale.TRIAL.value)])
-            builder.store(num_exec_time_ptr.type.pointee(0), num_exec_time_ptr)
+            for scale in (TimeScale.TRIAL, TimeScale.PASS, TimeScale.TIME_STEP):
+                num_exec_time_ptr = builder.gep(time_loc, [ctx.int32_ty(0),
+                                                           ctx.int32_ty(scale.value)])
+                builder.store(num_exec_time_ptr.type.pointee(0), num_exec_time_ptr)
 
         # Check if there's anything to reset
         for node in composition._all_nodes:
@@ -700,12 +880,13 @@ def gen_composition_exec(ctx, composition, *, tags:frozenset):
                 continue
 
             reinit_cond = cond_gen.generate_sched_condition(
-                builder, when, cond, node, None)
+                builder, when, cond, node, is_finished_callbacks, num_exec_locs)
             with builder.if_then(reinit_cond):
                 node_w = ctx.get_node_wrapper(composition, node)
                 node_reinit_f = ctx.import_llvm_function(node_w, tags=node_tags.union({"reset"}))
                 builder.call(node_reinit_f, [state, params, comp_in, data, data])
 
+        # Run controller if it's enabled in 'BEFORE' mode
         if simulation is False and composition.enable_controller and \
            composition.controller_mode == BEFORE:
             assert composition.controller is not None
@@ -716,43 +897,35 @@ def gen_composition_exec(ctx, composition, *, tags:frozenset):
 
 
         # Allocate run set structure
-        run_set_type = ir.ArrayType(ir.IntType(1), len(composition.nodes))
+        run_set_type = ir.ArrayType(ctx.bool_ty, len(composition.nodes))
         run_set_ptr = builder.alloca(run_set_type, name="run_set")
         builder.store(run_set_type(None), run_set_ptr)
 
-        # Allocate temporary output storage
-        output_storage = builder.alloca(data.type.pointee, name="output_storage")
 
         iter_ptr = builder.alloca(ctx.int32_ty, name="iter_counter")
         builder.store(ctx.int32_ty(0), iter_ptr)
 
-        # Generate pointers to 'is_finished' callbacks
-        is_finished_callbacks = {}
-        for node in composition.nodes:
-            args = [state, params, comp_in, data, output_storage]
-            wrapper = ctx.get_node_wrapper(composition, node)
-            is_finished_callbacks[node] = (wrapper, args)
-
+        # Start the main loop structure
         loop_condition = builder.append_basic_block(name="scheduling_loop_condition")
         builder.branch(loop_condition)
 
         # Generate a while not 'end condition' loop
         builder.position_at_end(loop_condition)
 
-        run_cond = cond_gen.generate_sched_condition(
+        trial_term_cond = cond_gen.generate_sched_condition(
             builder, composition.termination_processing[TimeScale.TRIAL],
-            cond, None, is_finished_callbacks)
-        run_cond = builder.not_(run_cond, name="not_run_cond")
+            cond, None, is_finished_callbacks, num_exec_locs)
+        trial_cond = builder.not_(trial_term_cond, name="not_trial_term_cond")
 
         loop_body = builder.append_basic_block(name="scheduling_loop_body")
-        exit_block = builder.append_basic_block(name="exit")
-        builder.cbranch(run_cond, loop_body, exit_block)
+        exit_block = builder.append_basic_block(name="trial_exit")
+        builder.cbranch(trial_cond, loop_body, exit_block)
 
         # Generate loop body
         builder.position_at_end(loop_body)
 
         zero = ctx.int32_ty(0)
-        any_cond = ir.IntType(1)(0)
+        any_cond = ctx.bool_ty(0)
         # Calculate execution set before running the mechanisms
         for idx, node in enumerate(composition.nodes):
             run_set_node_ptr = builder.gep(run_set_ptr,
@@ -760,22 +933,20 @@ def gen_composition_exec(ctx, composition, *, tags:frozenset):
                                            name="run_cond_ptr_" + node.name)
             node_cond = cond_gen.generate_sched_condition(
                 builder, composition._get_processing_condition_set(node),
-                cond, node, is_finished_callbacks)
+                cond, node, is_finished_callbacks, num_exec_locs)
             ran = cond_gen.generate_ran_this_pass(builder, cond, node)
             node_cond = builder.and_(node_cond, builder.not_(ran),
                                      name="run_cond_" + node.name)
             any_cond = builder.or_(any_cond, node_cond, name="any_ran_cond")
             builder.store(node_cond, run_set_node_ptr)
 
-        # Reset internal TIME_STEP and PASS clock for each node
+        # Reset internal TIME_STEP clock for each node
+        # NOTE: This is done _after_ condition evluation, otherwise
+        #       TIME_STEP related conditions will only see 0 executions
         for time_loc in num_exec_locs.values():
             num_exec_time_ptr = builder.gep(time_loc, [ctx.int32_ty(0),
                                                        ctx.int32_ty(TimeScale.TIME_STEP.value)])
             builder.store(num_exec_time_ptr.type.pointee(0), num_exec_time_ptr)
-            # FIXME: Move pass reset to actual pass count
-            num_exec_time_ptr = builder.gep(time_loc, [ctx.int32_ty(0),
-                                                       ctx.int32_ty(TimeScale.PASS.value)])
-            builder.store(num_exec_time_ptr.type.pointee(0), num_exec_time_ptr)
 
         for idx, node in enumerate(composition.nodes):
 
@@ -809,6 +980,8 @@ def gen_composition_exec(ctx, composition, *, tags:frozenset):
                 builder.store(builder.load(out_ptr), data_ptr)
 
         # Update step counter
+        # FIXME: This shouldn't really happen. If no node is run,
+        #        there's no change to the scheduling conditions and we hang.
         with builder.if_then(any_cond):
             builder.block.name = "inc_step"
             cond_gen.bump_ts(builder, cond)
@@ -829,6 +1002,11 @@ def gen_composition_exec(ctx, composition, *, tags:frozenset):
             builder.store(zero, iter_ptr)
             # Bumping automatically zeros lower elements
             cond_gen.bump_ts(builder, cond, (0, 1, 0))
+            # Reset internal PASS clock for each node
+            for time_loc in num_exec_locs.values():
+                num_exec_time_ptr = builder.gep(time_loc, [ctx.int32_ty(0),
+                                                           ctx.int32_ty(TimeScale.PASS.value)])
+                builder.store(num_exec_time_ptr.type.pointee(0), num_exec_time_ptr)
 
         builder.branch(loop_condition)
 
@@ -866,7 +1044,7 @@ def gen_composition_run(ctx, composition, *, tags:frozenset):
     for a in llvm_func.args:
         a.attributes.add('noalias')
 
-    state, params, data, data_in, data_out, runs_ptr, inputs_ptr = llvm_func.args
+    state, params, data, data_in, data_out, trials_ptr, inputs_ptr = llvm_func.args
     # simulation does not care about the output
     # it extracts results of the controller objective mechanism
     if simulation:
@@ -901,36 +1079,70 @@ def gen_composition_run(ctx, composition, *, tags:frozenset):
     cond_init = cond_type(cond_gen.get_condition_initializer())
     builder.store(cond_init, cond)
 
-    runs = builder.load(runs_ptr, "runs")
-    with helpers.for_loop_zero_inc(builder, runs, "run_loop") as (b, iters):
-        # Get the right input stimulus
-        input_idx = b.urem(iters, b.load(inputs_ptr))
-        data_in_ptr = b.gep(data_in, [input_idx])
+    trials = builder.load(trials_ptr, "trials")
+    iters_ptr = builder.alloca(trials.type)
+    builder.store(iters_ptr.type.pointee(0), iters_ptr)
 
-        # Reset internal clocks of each node
-        for idx, node in enumerate(composition._all_nodes):
-            #FIXME: This skips nested nodes
-            from psyneulink import Composition
-            if isinstance(node, Composition):
-                continue
-            node_state = builder.gep(state, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(idx)])
-            num_executions_ptr = helpers.get_state_ptr(builder, node, node_state, "num_executions")
-            num_exec_time_ptr = builder.gep(num_executions_ptr, [ctx.int32_ty(0), ctx.int32_ty(TimeScale.RUN.value)])
-            builder.store(num_exec_time_ptr.type.pointee(0), num_exec_time_ptr)
+    # Start the main loop structure
+    loop_condition = builder.append_basic_block(name="run_loop_condition")
+    builder.branch(loop_condition)
 
-        # Call execution
-        exec_tags = tags.difference({"run"})
-        exec_f = ctx.import_llvm_function(composition, tags=exec_tags)
-        b.call(exec_f, [state, params, data_in_ptr, data, cond])
+    # Generate a while not 'end condition' loop
+    builder.position_at_end(loop_condition)
 
-        if not simulation:
-            # Extract output_CIM result
-            idx = composition._get_node_index(composition.output_CIM)
-            result_ptr = b.gep(data, [ctx.int32_ty(0), ctx.int32_ty(0),
-                                      ctx.int32_ty(idx)])
-            output_ptr = b.gep(data_out, [iters])
-            result = b.load(result_ptr)
-            b.store(result, output_ptr)
+    run_term_cond = cond_gen.generate_sched_condition(
+        builder, composition.termination_processing[TimeScale.RUN],
+        cond, None, None, None)
+    run_cond = builder.not_(run_term_cond, name="not_run_term_cond")
+
+    # Iter cond
+    iters = builder.load(iters_ptr)
+    iter_cond = builder.icmp_unsigned("<", iters, trials)
+
+    # Increment. Use new name to not taint 'iters'
+    new_iters = builder.add(iters, iters.type(1))
+    builder.store(new_iters, iters_ptr)
+
+    loop_body = builder.append_basic_block(name="run_loop_body")
+    exit_block = builder.append_basic_block(name="run_exit")
+
+    run_cond = builder.and_(run_cond, iter_cond)
+    builder.cbranch(run_cond, loop_body, exit_block)
+
+    # Generate loop body
+    builder.position_at_end(loop_body)
+
+    # Get the right input stimulus
+    input_idx = builder.urem(iters, builder.load(inputs_ptr))
+    data_in_ptr = builder.gep(data_in, [input_idx])
+
+    # Reset internal 'RUN' clocks of each node
+    for idx, node in enumerate(composition._all_nodes):
+        node_state = builder.gep(state, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(idx)])
+        num_executions_ptr = helpers.get_state_ptr(builder, node, node_state, "num_executions")
+        num_exec_time_ptr = builder.gep(num_executions_ptr, [ctx.int32_ty(0), ctx.int32_ty(TimeScale.RUN.value)])
+        builder.store(num_exec_time_ptr.type.pointee(0), num_exec_time_ptr)
+
+    # Call execution
+    exec_tags = tags.difference({"run"})
+    exec_f = ctx.import_llvm_function(composition, tags=exec_tags)
+    builder.call(exec_f, [state, params, data_in_ptr, data, cond])
+
+    if not simulation:
+        # Extract output_CIM result
+        idx = composition._get_node_index(composition.output_CIM)
+        result_ptr = builder.gep(data, [ctx.int32_ty(0), ctx.int32_ty(0),
+                                        ctx.int32_ty(idx)])
+        output_ptr = builder.gep(data_out, [iters])
+        result = builder.load(result_ptr)
+        builder.store(result, output_ptr)
+
+    builder.branch(loop_condition)
+
+    # Exit
+    builder.position_at_end(exit_block)
+    # Store number of executed trials
+    builder.store(iters, trials_ptr)
 
     builder.ret_void()
     return llvm_func
@@ -953,7 +1165,7 @@ def gen_multirun_wrapper(ctx, function: ir.Function) -> ir.Function:
     # but hold entries for all parallel invocations.
     is_comp_run = len(function.args) == 7
     if is_comp_run:
-        runs_count = builder.load(multirun_f.args[5])
+        trials_count = builder.load(multirun_f.args[5])
         input_count = builder.load(multirun_f.args[6])
 
     with helpers.for_loop_zero_inc(builder, multi_runs, "multi_run_loop") as (b, index):
@@ -966,9 +1178,13 @@ def gen_multirun_wrapper(ctx, function: ir.Function) -> ir.Function:
                 # #runs and #trials needs to be the same for every invocation
                 if is_comp_run and i >= 5:
                     offset = ctx.int32_ty(0)
+                    # Reset trial count for every invocation.
+                    # Previous runs might have finished earlier
+                    if i == 5:
+                        builder.store(trials_count, arg)
                 # data arrays need special handling
                 elif is_comp_run and i == 4:  # data_out
-                    offset = b.mul(index, runs_count)
+                    offset = b.mul(index, trials_count)
                 elif is_comp_run and i == 3:  # data_in
                     offset = b.mul(index, input_count)
 
