@@ -166,6 +166,7 @@ class FuncExecution(CUDAExecution):
             Context(execution_id=eid) for eid in execution_ids
         ]
         self._component = component
+        self.__debug_env = debug_env
 
         par_struct_ty, ctx_struct_ty, vi_ty, vo_ty = self._bin_func.byref_arg_types
 
@@ -182,6 +183,11 @@ class FuncExecution(CUDAExecution):
         self._ct_vo = vo_ty()
         self._vi_ty = vi_ty
         self._vi_dty = _element_dtype(vi_ty)
+        if "stat" in self.__debug_env:
+            print("Input struct size:", _pretty_size(ctypes.sizeof(vi_ty)),
+                  "for", self._component.name)
+            print("Output struct size:", _pretty_size(ctypes.sizeof(vo_ty)),
+                  "for", self._component.name)
 
     def _get_compilation_param(self, name, initializer, arg, context):
         param = getattr(self._component._compilation_data, name)
@@ -191,6 +197,10 @@ class FuncExecution(CUDAExecution):
             struct_ty = self._bin_func.byref_arg_types[arg]
             struct = struct_ty(*initializer)
             param._set(struct, context=context)
+            if "stat" in self.__debug_env:
+                print("Instantiated struct:", name, "( size:" ,
+                      _pretty_size(ctypes.sizeof(struct_ty)), ")",
+                      "for", self._component.name)
 
         return struct
 
@@ -320,6 +330,10 @@ class CompExecution(CUDAExecution):
             cond_initializer = gen.get_condition_initializer()
             conds = cond_type(*cond_initializer)
             self._composition._compilation_data.scheduler_conditions._set(conds, context=self._execution_contexts[0])
+            if "stat" in self.__debug_env:
+                print("Instantiated condition struct ( size:" ,
+                      _pretty_size(ctypes.sizeof(struct_ty)), ")",
+                      "for", self._composition.name)
         return conds
 
     def _get_compilation_param(self, name, initializer, arg, context):
@@ -330,6 +344,10 @@ class CompExecution(CUDAExecution):
             struct_ty = self._bin_func.byref_arg_types[arg]
             struct = struct_ty(*initializer)
             param._set(struct, context=context)
+            if "stat" in self.__debug_env:
+                print("Instantiated struct:", name, "( size:" ,
+                      _pretty_size(ctypes.sizeof(struct_ty)), ")",
+                      "for", self._composition.name)
 
         return struct
 
@@ -463,6 +481,9 @@ class CompExecution(CUDAExecution):
         else:
             input_data = ([x] for x in self._composition._build_variable_for_input_CIM(inputs))
 
+        if "stat" in self.__debug_env:
+            print("Input struct size:", _pretty_size(ctypes.sizeof(c_input)),
+                  "for", self._composition.name)
         return c_input(*_tupleize(input_data))
 
     def freeze_values(self):
@@ -584,6 +605,8 @@ class CompExecution(CUDAExecution):
     def run(self, inputs, runs=0, num_input_sets=0):
         if isgenerator(inputs):
             inputs, runs = self._get_generator_run_input_struct(inputs, runs)
+            assert num_input_sets == 0 or num_input_sets == sys.maxsize
+            num_input_sets = len(inputs)
         else:
             inputs = self._get_run_input_struct(inputs, num_input_sets)
 
@@ -591,22 +614,35 @@ class CompExecution(CUDAExecution):
         if len(self._execution_contexts) > 1:
             ct_vo = ct_vo * len(self._execution_contexts)
         outputs = ct_vo()
+
+        if "stat" in self.__debug_env:
+            print("Input struct size:", _pretty_size(ctypes.sizeof(inputs)),
+                  "for", self._composition.name)
+            print("Output struct size:", _pretty_size(ctypes.sizeof(outputs)),
+                  "for", self._composition.name)
+
         runs_count = ctypes.c_int(runs)
         input_count = ctypes.c_int(num_input_sets)
         if len(self._execution_contexts) > 1:
             self._bin_run_multi_func.wrap_call(self._state_struct, self._param_struct,
                                                self._data_struct, inputs, outputs,
                                                runs_count, input_count, self._ct_len)
+            return _convert_ctype_to_python(outputs)
         else:
             self._bin_run_func.wrap_call(self._state_struct, self._param_struct,
                                          self._data_struct, inputs, outputs,
                                          runs_count, input_count)
-        return _convert_ctype_to_python(outputs)
+
+            # Extract only #trials elements in case the run exited early
+            assert runs_count.value <= runs, "Composition ran more times than allowed!"
+            return _convert_ctype_to_python(outputs)[0:runs_count.value]
 
     def cuda_run(self, inputs, runs, num_input_sets):
         # Create input buffer
         if isgenerator(inputs):
             inputs, runs = self._get_generator_run_input_struct(inputs, runs)
+            assert num_input_sets == 0 or num_input_sets == sys.maxsize
+            num_input_sets = len(inputs)
         else:
             inputs = self._get_run_input_struct(inputs, num_input_sets)
         data_in = self.upload_ctype(inputs, 'input')
@@ -618,10 +654,15 @@ class CompExecution(CUDAExecution):
         output_size = ctypes.sizeof(output_type)
         data_out = jit_engine.pycuda.driver.mem_alloc(output_size)
 
-        runs_count = jit_engine.pycuda.driver.In(np.int32(runs))
+        # number of trials argument
+        runs_np = np.array([runs] * len(self._execution_contexts), dtype=np.int32)
+        runs_count = jit_engine.pycuda.driver.InOut(runs_np)
+        self._uploaded_bytes['input'] += runs_np.nbytes
+        self._downloaded_bytes['input'] += runs_np.nbytes
+
+        # input_count argument
         input_count = jit_engine.pycuda.driver.In(np.int32(num_input_sets))
-        # runs_count + input_count
-        self._uploaded_bytes['input'] += 8
+        self._uploaded_bytes['input'] += 4
 
         self._bin_run_func.cuda_call(self._cuda_state_struct,
                                      self._cuda_param_struct,
@@ -631,7 +672,12 @@ class CompExecution(CUDAExecution):
 
         # Copy the data struct from the device
         ct_out = self.download_ctype(data_out, output_type, 'result')
-        return _convert_ctype_to_python(ct_out)
+        if len(self._execution_contexts) > 1:
+            return _convert_ctype_to_python(ct_out)
+        else:
+            # Extract only #trials elements in case the run exited early
+            assert runs_np[0] <= runs, "Composition ran more times than allowed!"
+            return _convert_ctype_to_python(ct_out)[0:runs_np[0]]
 
     def cuda_evaluate(self, variable, search_space):
         ocm = self._composition.controller
