@@ -344,7 +344,10 @@ class ConditionSet(object):
 
     """
     def __init__(self, conditions=None):
-        self.conditions = conditions if conditions is not None else {}
+        self.conditions = {}
+
+        if conditions is not None:
+            self.add_condition_set(conditions)
 
     def __contains__(self, item):
         return item in self.conditions
@@ -717,9 +720,9 @@ class Not(Condition):
     def __init__(self, condition):
         self.condition = condition
 
-        def inner_func(*args, **kwargs):
+        def inner_func(condition, *args, **kwargs):
             return not condition.is_satisfied(*args, **kwargs)
-        super().__init__(inner_func)
+        super().__init__(inner_func, condition)
 
     @Condition.owner.setter
     def owner(self, value):
@@ -1240,6 +1243,8 @@ class BeforeNCalls(_DependencyValidation, Condition):
 
     """
     def __init__(self, dependency, n, time_scale=TimeScale.TRIAL):
+        self.time_scale = time_scale
+
         def func(dependency, n, scheduler=None, execution_id=None):
             try:
                 num_calls = scheduler.counts_total[execution_id][time_scale][dependency]
@@ -1276,6 +1281,8 @@ class AtNCalls(_DependencyValidation, Condition):
 
     """
     def __init__(self, dependency, n, time_scale=TimeScale.TRIAL):
+        self.time_scale = time_scale
+
         def func(dependency, n, scheduler=None, execution_id=None):
             try:
                 num_calls = scheduler.counts_total[execution_id][time_scale][dependency]
@@ -1336,6 +1343,8 @@ class AfterNCalls(_DependencyValidation, Condition):
 
     """
     def __init__(self, dependency, n, time_scale=TimeScale.TRIAL):
+        self.time_scale = time_scale
+
         def func(dependency, n, scheduler=None, execution_id=None):
             try:
                 num_calls = scheduler.counts_total[execution_id][time_scale][dependency]
@@ -1479,6 +1488,8 @@ class AllHaveRun(_DependencyValidation, Condition):
 
     """
     def __init__(self, *dependencies, time_scale=TimeScale.TRIAL):
+        self.time_scale = time_scale
+
         def func(*dependencies, scheduler=None, execution_id=None):
             if len(dependencies) == 0:
                 dependencies = scheduler.nodes
