@@ -1154,24 +1154,21 @@ class DDM(ProcessingMechanism):
 
         return mech_out, builder
 
-    @handle_external_context(execution_id=NotImplemented)
-    def reset(self, *args, force=False, context=None):
+    @handle_external_context(fallback_most_recent=True)
+    def reset(self, *args, force=False, context=None, **kwargs):
         from psyneulink.core.components.functions.statefulfunctions.integratorfunctions import IntegratorFunction
 
-        if context.execution_id is NotImplemented:
-            context.execution_id = self.most_recent_context.execution_id
-
         # (1) reset function, (2) update mechanism value, (3) update output ports
         if isinstance(self.function, IntegratorFunction):
-            new_values = self.function.reset(*args, context=context)
-            self.parameters.value._set(np.array(new_values), context)
+            new_values = self.function.reset(*args, **kwargs, context=context)
+            self.parameters.value._set(convert_to_np_array(new_values), context)
             self._update_output_ports(context=context)
 
     @handle_external_context()
     def is_finished(self, context=None):
         # find the single numeric entry in previous_value
         try:
-            single_value = self.function.get_previous_value(context)
+            single_value = self.function.parameters.previous_value._get(context)
         except AttributeError:
             # Analytical function so it is always finished after it is called
             return True
@@ -1207,7 +1204,7 @@ class DDM(ProcessingMechanism):
         try:
             prev_val_ptr = pnlvm.helpers.get_state_ptr(builder, self.function, func_state_ptr, "previous_value")
         except ValueError:
-            return pnlvm.ir.IntType(1)(1)
+            return ctx.bool_ty(1)
 
         # Extract scalar value from ptr
         prev_val_ptr = builder.gep(prev_val_ptr, [ctx.int32_ty(0), ctx.int32_ty(0), ctx.int32_ty(0)])
