@@ -193,6 +193,7 @@ from collections.abc import Iterable
 from psyneulink.core import llvm as pnlvm
 from psyneulink.core.components.component import _get_parametervalue_attr
 from psyneulink.core.components.functions.function import Function, get_matrix, is_function_type
+from psyneulink.core.components.functions.statefulfunctions.integratorfunctions import AdaptiveIntegrator
 from psyneulink.core.components.functions.learningfunctions import Hebbian
 from psyneulink.core.components.functions.objectivefunctions import Stability
 from psyneulink.core.components.functions.combinationfunctions import LinearCombination
@@ -614,6 +615,7 @@ class RecurrentTransferMechanism(TransferMechanism):
         matrix = Parameter(HOLLOW_MATRIX, modulable=True, getter=_recurrent_transfer_mechanism_matrix_getter, setter=_recurrent_transfer_mechanism_matrix_setter)
         auto = Parameter(1, modulable=True)
         hetero = Parameter(0, modulable=True)
+        integrator_function = Parameter(AdaptiveIntegrator, stateful=False, loggable=False, dependencies='combination_function')
         combination_function = Parameter(LinearCombination, stateful=False, loggable=False)
         smoothing_factor = Parameter(0.5, modulable=True)
         enable_learning = False
@@ -966,7 +968,7 @@ class RecurrentTransferMechanism(TransferMechanism):
 
             # creating a recurrent_projection changes the default variable shape
             # so we have to reshape any Paramter Functions
-            self._update_parameter_class_variables(context)
+            self._update_default_variable(self.defaults.variable, context)
 
         self.aux_components.append(self.recurrent_projection)
 
@@ -1108,7 +1110,7 @@ class RecurrentTransferMechanism(TransferMechanism):
                                         learning_rate,
                                         learning_condition,
                                         matrix,
-                                        context=None):
+                                        ):
 
         learning_mechanism = AutoAssociativeLearningMechanism(default_variable=copy.deepcopy([activity_vector.defaults.value]),
                                                               # learning_signals=[self.recurrent_projection],
@@ -1175,14 +1177,23 @@ class RecurrentTransferMechanism(TransferMechanism):
         try:
             shared_params = self.initial_shared_parameters['learning_mechanism']
 
-            if learning_condition is None:
-                learning_condition = shared_params['learning_condition']
+            try:
+                if learning_condition is None:
+                    learning_condition = shared_params['learning_condition']
+            except KeyError:
+                pass
 
-            if learning_rate is None:
-                learning_rate = shared_params['learning_rate']
+            try:
+                if learning_rate is None:
+                    learning_rate = shared_params['learning_rate']
+            except KeyError:
+                pass
 
-            if learning_function is None:
-                learning_function = shared_params['learning_function']
+            try:
+                if learning_function is None:
+                    learning_function = shared_params['learning_function']
+            except KeyError:
+                pass
         except KeyError:
             pass
 
@@ -1191,7 +1202,7 @@ class RecurrentTransferMechanism(TransferMechanism):
                                                                        learning_rate=learning_rate,
                                                                        learning_condition=learning_condition,
                                                                        matrix=self.recurrent_projection,
-                                                                       context=context)
+                                                                       )
 
         self.learning_projection = self.learning_mechanism.output_ports[LEARNING_SIGNAL].efferents[0]
         if self.learning_mechanism is None:
@@ -1206,10 +1217,14 @@ class RecurrentTransferMechanism(TransferMechanism):
         return super()._execute(variable, context, runtime_params)
 
     def _parse_function_variable(self, variable, context=None):
+        variable = self._parse_integrator_function_variable(variable, context=context)
+        return super()._parse_function_variable(variable, context=context)
+
+    def _parse_integrator_function_variable(self, variable, context=None):
         if self.has_recurrent_input_port:
             variable = self.combination_function.execute(variable=variable, context=context)
 
-        return super()._parse_function_variable(variable, context=context)
+        return variable
 
     def _get_variable_from_input(self, input, context=None):
         if self.has_recurrent_input_port:
@@ -1223,11 +1238,6 @@ class RecurrentTransferMechanism(TransferMechanism):
 
         return super()._get_variable_from_input(input, context)
 
-    @handle_external_context(execution_id=NotImplemented)
-    def reset(self, *args, force=False, context=None):
-        super().reset(*args, force=force, context=context)
-        self.parameters.value.clear_history(context)
-
     @property
     def _learning_signal_source(self):
         """Return default source of learning signal (`Primary OutputPort <OutputPort_Primary>)`
