@@ -366,4 +366,4 @@ if args.enable_plot:
     plt.title('stimulus onset asynchrony - horse race model ')
     plt.legend(['congruent', 'incongruent', 'neutral'])
     plt.ylabel('reaction time in ms')
-    plt.show()
+    plt.show(block=not pnl._called_from_pytest)
