@@ -11,14 +11,14 @@ import psyneulink.core.llvm as pnlvm
 
 class TestBinaryOperations:
     @pytest.mark.parametrize("param1, param2", [
-                        (1, 2), # scalar - scalar
-                        (np.ones(2), 2), # vec - scalar
-                        (2, np.ones(2)), # scalar - vec
-                        (np.ones((2, 2)), 2), # mat - scalar
-                        (2, np.ones((2, 2))), # scalar - mat
-                        (np.ones(2), np.array([1, 2])), # vec - vec
-                        (np.ones((2, 2)), np.array([[1, 2], [3, 4]])), # mat - mat
-                        ])
+                        (1, 2),
+                        (np.ones(2), 2),
+                        (2, np.ones(2)),
+                        (np.ones((2, 2)), 2),
+                        (2, np.ones((2, 2))),
+                        (np.ones(2), np.array([1, 2])),
+                        (np.ones((2, 2)), np.array([[1, 2], [3, 4]])),
+                        ], ids=["scalar-scalar", "vec-scalar", "scalar-vec", "mat-scalar", "scalar-mat", "vec-vec", "mat-mat"])
     @pytest.mark.parametrize("bin_execute", ['Python',
                                              pytest.param('LLVM', marks=pytest.mark.llvm),
                                              pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
@@ -41,14 +41,15 @@ class TestBinaryOperations:
         assert np.allclose(val, param1 + param2)
 
     @pytest.mark.parametrize("param1, param2", [
-                        (1, 2), # scalar - scalar
-                        (np.ones(2), 2), # vec - scalar
-                        (2, np.ones(2)), # scalar - vec
-                        (np.ones((2, 2)), 2), # mat - scalar
-                        (2, np.ones((2, 2))), # scalar - mat
-                        (np.ones(2), np.array([1, 2])), # vec - vec
-                        (np.ones((2, 2)), np.array([[1, 2], [3, 4]])), # mat - mat
-                        ])
+                        (1, 2),
+                        (np.ones(2), 2),
+                        (2, np.ones(2)),
+                        (np.ones((2, 2)), 2),
+                        (2, np.ones((2, 2))),
+                        (np.ones(2), np.array([1, 2])),
+                        (np.ones(2), np.array([2.])),
+                        (np.ones((2, 2)), np.array([[1, 2], [3, 4]])),
+                        ], ids=["scalar-scalar", "vec-scalar", "scalar-vec", "mat-scalar", "scalar-mat", "vec-vec", "vec-vec-differing", "mat-mat"])
     @pytest.mark.parametrize("bin_execute", ['Python',
                                              pytest.param('LLVM', marks=pytest.mark.llvm),
                                              pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
@@ -70,6 +71,37 @@ class TestBinaryOperations:
         val = benchmark(e, 0)
         assert np.allclose(val, param1 * param2)
 
+    @pytest.mark.parametrize("param1, param2", [
+                    (1, 2),
+                    (np.ones(2), 2),
+                    (2, np.ones(2)),
+                    (np.ones((2, 2)), 2),
+                    (2, np.ones((2, 2))),
+                    (np.ones(2), np.array([1, 2])),
+                    (np.ones(2), np.array([2.])),
+                    (np.ones((2, 2)), np.array([[1, 2], [3, 4]])),
+                    ], ids=["scalar-scalar", "vec-scalar", "scalar-vec", "mat-scalar", "scalar-mat", "vec-vec", "vec-vec-differing", "mat-mat"])
+    @pytest.mark.parametrize("bin_execute", ['Python',
+                                             pytest.param('LLVM', marks=pytest.mark.llvm),
+                                             pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
+                                            ])
+    @pytest.mark.benchmark(group="Function UDF")
+    def test_user_def_func_div(self, param1, param2, bin_execute, benchmark):
+        # default val is same shape as expected output
+        def myFunction(_, param1, param2):
+            # we only use param1 and param2 to avoid automatic shape changes of the variable
+            return param1 / param2
+
+        U = UserDefinedFunction(custom_function=myFunction, param1=param1, param2=param2)
+        if bin_execute == 'LLVM':
+            e = pnlvm.execution.FuncExecution(U).execute
+        elif bin_execute == 'PTX':
+            e = pnlvm.execution.FuncExecution(U).cuda_execute
+        else:
+            e = U
+        val = benchmark(e, 0)
+        assert np.allclose(val, np.divide(param1, param2))
+
     @pytest.mark.parametrize("op", [ # parameter is string since compiled udf doesn't support closures as of present
                         "AND",
                         "OR",
@@ -109,72 +141,59 @@ class TestBinaryOperations:
         val = benchmark(e, [0])
         assert val == 1.0
 
-    @pytest.mark.parametrize("op", [ # parameter is string since compiled udf doesn't support closures as of present
-                        "Eq",
-                        "NotEq",
-                        "Lt",
-                        "LtE",
-                        "Gt",
-                        "GtE",
+    @pytest.mark.parametrize("op,var1,var2,expected", [ # parameter is string since compiled udf doesn't support closures as of present
+                        ("Eq", 1.0, 2.0, 0.0),
+                        ("NotEq", 1.0, 2.0, 1.0),
+                        ("Lt", 1.0, 2.0, 1.0),
+                        ("LtE", 1.0, 2.0, 1.0),
+                        ("Gt", 1.0, 2.0, 0.0),
+                        ("GtE", 1.0, 2.0, 0.0),
                         ])
     @pytest.mark.parametrize("bin_execute", ['Python',
                                              pytest.param('LLVM', marks=pytest.mark.llvm),
                                              pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
                                             ])
     @pytest.mark.benchmark(group="Function UDF")
-    def test_user_def_func_cmpop(self, op, bin_execute, benchmark):
+    def test_user_def_func_cmpop(self, op, var1, var2, expected, bin_execute, benchmark):
+        # we explicitly use np here to ensure that the result is castable to float in the scalar-scalar case
         if op == "Eq":
-            def myFunction(variable):
-                var1 = 1.0
-                var2 = 1.0
+            def myFunction(variable, var1, var2):
                 if var1 == var2:
                     return 1.0
                 else:
                     return 0.0
         elif op == "NotEq":
-            def myFunction(variable):
-                var1 = 1.0
-                var2 = 2.0
+            def myFunction(variable, var1, var2):
                 if var1 != var2:
                     return 1.0
                 else:
                     return 0.0
         elif op == "Lt":
-            def myFunction(variable):
-                var1 = 1.0
-                var2 = 2.0
+            def myFunction(variable, var1, var2):
                 if var1 < var2:
                     return 1.0
                 else:
                     return 0.0
         elif op == "LtE":
-            def myFunction(variable):
-                var1 = 1.0
-                var2 = 2.0
-                var3 = 1.0
-                if var1 <= var2 and var1 <= var3:
+            def myFunction(variable, var1, var2):
+                if var1 <= var2:
                     return 1.0
                 else:
                     return 0.0
         elif op == "Gt":
-            def myFunction(variable):
-                var1 = 2.0
-                var2 = 1.0
+            def myFunction(variable, var1, var2):
                 if var1 > var2:
                     return 1.0
                 else:
                     return 0.0
         elif op == "GtE":
-            def myFunction(variable):
-                var1 = 3.0
-                var2 = 2.0
-                var3 = 3.0
-                if var1 >= var2 and var1 >= var3:
+            def myFunction(variable, var1, var2):
+                if var1 >= var2:
                     return 1.0
                 else:
                     return 0.0
 
-        U = UserDefinedFunction(custom_function=myFunction, default_variable=[0])
+        U = UserDefinedFunction(custom_function=myFunction, default_variable=[0], var1=var1, var2=var2)
         if bin_execute == 'LLVM':
             e = pnlvm.execution.FuncExecution(U).execute
         elif bin_execute == 'PTX':
@@ -182,7 +201,87 @@ class TestBinaryOperations:
         else:
             e = U
         val = benchmark(e, [0])
-        assert val == 1.0
+        assert np.allclose(expected, val)
+
+    @pytest.mark.parametrize("op,var1,var2,expected", [ # parameter is string since compiled udf doesn't support closures as of present
+                        ("Eq", 1.0, 2.0, 0.0),
+                        ("Eq", [1.0, 2.0], [1.0, 2.0], [1.0, 1.0]),
+                        ("Eq", 1.0, [1.0, 2.0], [1.0, 0.0]),
+                        ("Eq", [2.0, 1.0], 1.0, [0.0, 1.0]),
+                        ("Eq", [[1.0, 2.0], [3.0, 4.0]], 1.0, [[1.0, 0.0], [0.0, 0.0]]),
+                        ("Eq", 1.0, [[1.0, 2.0], [3.0, 4.0]], [[1.0, 0.0], [0.0, 0.0]]),
+                        ("Eq", [[1.0, 2.0], [3.0, 4.0]], [[1.0, 2.0], [3.0, 4.0]], [[1.0, 1.0], [1.0, 1.0]]),
+                        ("NotEq", 1.0, 2.0, 1.0),
+                        ("NotEq", [1.0, 2.0], [1.0, 2.0], [0.0, 0.0]),
+                        ("NotEq", 1.0, [1.0, 2.0], [0.0, 1.0]),
+                        ("NotEq", [2.0, 1.0], 1.0, [1.0, 0.0]),
+                        ("NotEq", [[1.0, 2.0], [3.0, 4.0]], 1.0, [[0.0, 1.0], [1.0, 1.0]]),
+                        ("NotEq", 1.0, [[1.0, 2.0], [3.0, 4.0]], [[0.0, 1.0], [1.0, 1.0]]),
+                        ("NotEq", [[1.0, 2.0], [3.0, 4.0]], [[1.0, 2.0], [3.0, 4.0]], [[0.0, 0.0], [0.0, 0.0]]),
+                        ("Lt", 1.0, 2.0, 1.0),
+                        ("Lt", [1.0, 2.0], [1.0, 2.0], [0.0, 0.0]),
+                        ("Lt", 1.0, [1.0, 2.0], [0.0, 1.0]),
+                        ("Lt", [2.0, 1.0], 1.0, [0.0, 0.0]),
+                        ("Lt", [[1.0, 2.0], [3.0, 4.0]], 1.0, [[0.0, 0.0], [0.0, 0.0]]),
+                        ("Lt", 1.0, [[1.0, 2.0], [3.0, 4.0]], [[0.0, 1.0], [1.0, 1.0]]),
+                        ("Lt", [[1.0, 2.0], [3.0, 4.0]], [[1.0, 2.0], [3.0, 4.0]], [[0.0, 0.0], [0.0, 0.0]]),
+                        ("LtE", 1.0, 2.0, 1.0),
+                        ("LtE", [1.0, 2.0], [1.0, 2.0], [1.0, 1.0]),
+                        ("LtE", 1.0, [1.0, 2.0], [1.0, 1.0]),
+                        ("LtE", [2.0, 1.0], 1.0, [0.0, 1.0]),
+                        ("LtE", [[1.0, 2.0], [3.0, 4.0]], 1.0, [[1.0, 0.0], [0.0, 0.0]]),
+                        ("LtE", 1.0, [[1.0, 2.0], [3.0, 4.0]], [[1.0, 1.0], [1.0, 1.0]]),
+                        ("LtE", [[1.0, 2.0], [3.0, 4.0]], [[1.0, 2.0], [3.0, 4.0]], [[1.0, 1.0], [1.0, 1.0]]),
+                        ("Gt", 1.0, 2.0, 0.0),
+                        ("Gt", [1.0, 2.0], [1.0, 2.0], [0.0, 0.0]),
+                        ("Gt", 1.0, [1.0, 2.0], [0.0, 0.0]),
+                        ("Gt", [2.0, 1.0], 1.0, [1.0, 0.0]),
+                        ("Gt", [[1.0, 2.0], [3.0, 4.0]], 1.0, [[0.0, 1.0], [1.0, 1.0]]),
+                        ("Gt", 1.0, [[1.0, 2.0], [3.0, 4.0]], [[0.0, 0.0], [0.0, 0.0]]),
+                        ("Gt", [[1.0, 2.0], [3.0, 4.0]], [[1.0, 2.0], [3.0, 4.0]], [[0.0, 0.0], [0.0, 0.0]]),
+                        ("GtE", 1.0, 2.0, 0.0),
+                        ("GtE", [1.0, 2.0], [1.0, 2.0], [1.0, 1.0]),
+                        ("GtE", 1.0, [1.0, 2.0], [1.0, 0.0]),
+                        ("GtE", [2.0, 1.0], 1.0, [1.0, 1.0]),
+                        ("GtE", [[1.0, 2.0], [3.0, 4.0]], 1.0, [[1.0, 1.0], [1.0, 1.0]]),
+                        ("GtE", 1.0, [[1.0, 2.0], [3.0, 4.0]], [[1.0, 0.0], [0.0, 0.0]]),
+                        ("GtE", [[1.0, 2.0], [3.0, 4.0]], [[1.0, 2.0], [3.0, 4.0]], [[1.0, 1.0], [1.0, 1.0]]),
+                        ])
+    @pytest.mark.parametrize("bin_execute", ['Python',
+                                             pytest.param('LLVM', marks=pytest.mark.llvm),
+                                             pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
+                                            ])
+    @pytest.mark.benchmark(group="Function UDF")
+    def test_user_def_func_cmpop_numpy(self, op, var1, var2, expected, bin_execute, benchmark):
+        # we explicitly use np here to ensure that the result is castable to float in the scalar-scalar case
+        if op == "Eq":
+            def myFunction(variable, var1, var2):
+                return np.equal(var1, var2).astype(float)
+        elif op == "NotEq":
+            def myFunction(variable, var1, var2):
+                return np.not_equal(var1, var2).astype(float)
+        elif op == "Lt":
+            def myFunction(variable, var1, var2):
+                return np.less(var1, var2).astype(float)
+        elif op == "LtE":
+            def myFunction(variable, var1, var2):
+                return np.less_equal(var1, var2).astype(float)
+        elif op == "Gt":
+            def myFunction(variable, var1, var2):
+                return np.greater(var1, var2).astype(float)
+        elif op == "GtE":
+            def myFunction(variable, var1, var2):
+                return np.greater_equal(var1, var2).astype(float)
+
+        U = UserDefinedFunction(custom_function=myFunction, default_variable=[0], var1=var1, var2=var2)
+        if bin_execute == 'LLVM':
+            e = pnlvm.execution.FuncExecution(U).execute
+        elif bin_execute == 'PTX':
+            e = pnlvm.execution.FuncExecution(U).cuda_execute
+        else:
+            e = U
+        val = benchmark(e, [0])
+        assert np.allclose(expected, val)
 
 class TestUserDefFunc:
 
@@ -251,10 +350,10 @@ class TestUserDefFunc:
         assert np.allclose(val, [[6, 10]])
 
     @pytest.mark.parametrize("variable", [
-                        (1), # scalar
-                        (np.ones((2))), # vec-2d
-                        (np.ones((2, 2))) # mat
-                        ])
+                        (1),
+                        (np.ones((2))),
+                        (np.ones((2, 2)))
+                        ], ids=["scalar", "vec-2d", "mat"])
     @pytest.mark.parametrize("bin_execute", ['Python',
                                              pytest.param('LLVM', marks=pytest.mark.llvm),
                                              pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
@@ -274,6 +373,25 @@ class TestUserDefFunc:
         val = benchmark(e, variable)
         assert np.allclose(val, -variable)
 
+    @pytest.mark.parametrize("bin_execute", ['Python',
+                                             pytest.param('LLVM', marks=pytest.mark.llvm),
+                                             pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
+                                            ])
+    @pytest.mark.benchmark(group="Function UDF")
+    def test_user_def_reward_func(self, bin_execute, benchmark):
+        variable = [[1,2,3,4]]
+        def myFunction(x,t0=0.48):
+            return (x[0][0]>0).astype(float) * (x[0][2]>0).astype(float) / (np.max([x[0][1],x[0][3]]) + t0)
+        U = UserDefinedFunction(custom_function=myFunction, default_variable=variable, param=variable)
+        if bin_execute == 'LLVM':
+            e = pnlvm.execution.FuncExecution(U).execute
+        elif bin_execute == 'PTX':
+            e = pnlvm.execution.FuncExecution(U).cuda_execute
+        else:
+            e = U
+        val = benchmark(e, variable)
+        assert np.allclose(val, 0.2232142857142857)
+
     @pytest.mark.parametrize("dtype, expected", [ # parameter is string since compiled udf doesn't support closures as of present
                         ("SCALAR", 1.0),
                         ("VECTOR", [1,2]),
@@ -318,25 +436,47 @@ class TestUserDefFunc:
         val = benchmark(e, 0)
         assert np.allclose(val, expected)
 
-    @pytest.mark.parametrize("op,expected", [ # parameter is string since compiled udf doesn't support closures as of present
-                    ("TANH", [0.76159416, 0.99505475]),
-                    ("EXP", [2.71828183, 20.08553692]),
+    @pytest.mark.parametrize("op,variable,expected", [ # parameter is string since compiled udf doesn't support closures as of present
+                    ("TANH", [[1, 3]], [0.76159416, 0.99505475]),
+                    ("EXP", [[1, 3]], [2.71828183, 20.08553692]),
+                    ("SHAPE", [1, 2], [2]),
+                    ("SHAPE", [[1, 3]], [1, 2]),
+                    ("ASTYPE_FLOAT", [1], [1.0]),
+                    ("ASTYPE_INT", [-1.5], [-1.0]),
+                    ("NP_MAX", [0.0, 0.0], 0),
+                    ("NP_MAX", [1.0, 2.0], 2),
+                    ("NP_MAX", [[2.0, 1.0], [6.0, 2.0]], 6),
+                    ("FLATTEN", [[1.0, 2.0], [3.0, 4.0]], [1.0, 2.0, 3.0, 4.0])
                     ])
     @pytest.mark.parametrize("bin_execute", ['Python',
                                              pytest.param('LLVM', marks=pytest.mark.llvm),
                                              pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
                                             ])
     @pytest.mark.benchmark(group="Function UDF")
-    def test_user_def_func_numpy(self, op, expected, bin_execute, benchmark):
-        variable = [[1, 3]]
+    def test_user_def_func_numpy(self, op, variable, expected, bin_execute, benchmark):
         if op == "TANH":
             def myFunction(variable):
                 return np.tanh(variable)
         elif op == "EXP":
             def myFunction(variable):
                 return np.exp(variable)
-
-        U = UserDefinedFunction(custom_function=myFunction, default_variable=[[0, 0]])
+        elif op == "SHAPE":
+            def myFunction(variable):
+                return variable.shape
+        elif op == "ASTYPE_FLOAT":
+            def myFunction(variable):
+                return variable.astype(float)
+        elif op == "ASTYPE_INT":
+            # return types cannot be integers, so we cast back to float and check for truncation
+            def myFunction(variable):
+                return variable.astype(int).astype(float)
+        elif op == "NP_MAX":
+            def myFunction(variable):
+                return np.max(variable)
+        elif op == "FLATTEN":
+            def myFunction(variable):
+                return variable.flatten()
+        U = UserDefinedFunction(custom_function=myFunction, default_variable=variable)
         if bin_execute == 'LLVM':
             e = pnlvm.execution.FuncExecution(U).execute
         elif bin_execute == 'PTX':
@@ -367,6 +507,47 @@ class TestUserDefFunc:
         val = benchmark(e, [-1, 2, 3, 4])
         assert np.allclose(val, [[10]])
 
+    @pytest.mark.parametrize("op,variable,expected", [ # parameter is string since compiled udf doesn't support closures as of present
+                    ("SUM", [1.0, 3.0], 4),
+                    ("SUM", [[1.0], [3.0]], [4.0]),
+                    ("LEN", [1.0, 3.0], 2),
+                    ("LEN", [[1.0], [3.0]], 2),
+                    ("LEN_TUPLE", [0, 0], 2),
+                    ("MAX_MULTI", [1,], 6),
+                    ("MAX", [1.0, 3.0, 2.0], 3.0),
+                    ])
+    @pytest.mark.parametrize("bin_execute", ['Python',
+                                             pytest.param('LLVM', marks=pytest.mark.llvm),
+                                             pytest.param('PTX', marks=[pytest.mark.llvm, pytest.mark.cuda]),
+                                            ])
+    @pytest.mark.benchmark(group="Function UDF")
+    def test_user_def_func_builtin(self, op, variable, expected, bin_execute, benchmark):
+        if op == "SUM":
+            def myFunction(variable):
+                return sum(variable)
+        elif op == "LEN":
+            def myFunction(variable):
+                return len(variable)
+        elif op == "LEN_TUPLE":
+            def myFunction(variable):
+                return len((1,2))
+        elif op == "MAX":
+            def myFunction(variable):
+                return max(variable)
+        elif op == "MAX_MULTI":
+            # special cased, since passing in multiple variables without a closure is hard
+            def myFunction(_):
+                return max(1, 2, 3, 4, 5, 6, -1, -2)
+
+        U = UserDefinedFunction(custom_function=myFunction, default_variable=variable)
+        if bin_execute == 'LLVM':
+            e = pnlvm.execution.FuncExecution(U).execute
+        elif bin_execute == 'PTX':
+            e = pnlvm.execution.FuncExecution(U).cuda_execute
+        else:
+            e = U
+        val = benchmark(e, variable)
+        assert np.allclose(val, expected)
 
     @pytest.mark.parametrize("bin_execute", ['Python',
                                              pytest.param('LLVM', marks=pytest.mark.llvm),
