@@ -1002,6 +1002,15 @@ class TestLog:
             assert log_dict['Run'] == [[0], [0], [0], [1], [1], [1]]
             assert np.allclose(log_dict['value'], [[[0.52466739, 0.47533261]] * 6])
 
+    def test_log_with_non_full_execution_id_entries(self):
+        t = pnl.TransferMechanism()
+
+        t.parameters.noise.set(0, context=1, override=True)
+        t.parameters.value.set(0, context=2, override=True)
+
+        t.log.nparray()
+        t.log.nparray_dictionary()
+
 
 class TestClearLog:
 
