@@ -194,7 +194,7 @@ class TestTransferMechanismNoise:
         )
         T.reset_stateful_function_when = Never()
         val = T.execute([0, 0, 0, 0])
-        assert np.allclose(val, [[0.90104733, 0.95869664, -0.81762271, -1.39225086]])
+        assert np.allclose(val, [[-0.6931771, 1.00018003, 2.5496904, -0.71562799]])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
@@ -210,7 +210,7 @@ class TestTransferMechanismNoise:
         )
         T.reset_stateful_function_when = Never()
         val = T.execute([0, 0, 0, 0])
-        expected = [0.6202001216069017, 0.840166034615641, 0.7279826246296707, -1.5678942459349325]
+        expected = [[-1.56404341, -3.01320403, -1.22503678, 1.3093712]]
         assert np.allclose(np.asfarray(val[0]), expected)
 
     @pytest.mark.mechanism
@@ -293,7 +293,7 @@ class TestDistributionFunctions:
         )
 
         val = T.execute([0, 0, 0, 0])
-        assert np.allclose(val, [[0.90104733, 0.95869664, -0.81762271, -1.39225086]])
+        assert np.allclose(val, [[-0.6931771 ,  1.00018003,  2.5496904 , -0.71562799]])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
@@ -325,7 +325,7 @@ class TestDistributionFunctions:
             integrator_mode=True,
         )
         val = T.execute([0, 0, 0, 0])
-        assert np.allclose(val, [[0.7025651, 0.33105989, 1.40978493, 0.9633011]])
+        assert np.allclose(val, [[1.53154485, 0.36141864, 0.64740347, 0.87558564]])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
@@ -368,7 +368,7 @@ class TestDistributionFunctions:
             integrator_mode=True
         )
         val = T.execute([0, 0, 0, 0])
-        assert np.allclose(val, [[0.50468686, 0.28183784, 0.7558042, 0.618369]])
+        assert np.allclose(val, [[0.78379859, 0.30331273, 0.47659695, 0.58338204]])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
@@ -383,7 +383,7 @@ class TestDistributionFunctions:
             integrator_mode=True
         )
         val = T.execute([0, 0, 0, 0])
-        assert np.allclose(val, [[0.7025651, 0.33105989, 1.40978493, 0.9633011]])
+        assert np.allclose(val, [[1.53154485, 0.36141864, 0.64740347, 0.87558564]])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
@@ -398,7 +398,7 @@ class TestDistributionFunctions:
             integrator_mode=True
         )
         val = T.execute([0, 0, 0, 0])
-        assert np.allclose(val, [[1.0678432, 0.34512569, 1.07265769, 1.3550318]])
+        assert np.allclose(val, [[0.39640095, 0.45094588, 2.88271841, 0.41203028]])
 
 
 class TestTransferMechanismFunctions:
@@ -1190,7 +1190,7 @@ class TestTransferMechanismTimeConstant:
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
     def test_transfer_mech_integration_rate_2(self):
-        with pytest.raises(TransferError) as error_text:
+        with pytest.raises(ParameterError) as error_text:
             T = TransferMechanism(
                 name='T',
                 default_variable=[0, 0, 0, 0],
@@ -1200,7 +1200,7 @@ class TestTransferMechanismTimeConstant:
             )
             T.execute([1, 1, 1, 1])
         assert (
-            "Value(s) in \'integration_rate\' arg for" in str(error_text.value)
+            "'integration_rate'" in str(error_text.value)
             and "must be an int or float in the interval [0,1]" in str(error_text.value)
         )
 
@@ -1563,7 +1563,7 @@ class TestTransferMechanismMultipleInputPorts:
             default_variable=[[0.0, 0.0], [0.0, 0.0]]
         )
         val = T.execute([[1.0, 2.0], [3.0, 4.0]])
-        assert np.allclose(val, [[4.80209467, 6.91739329], [5.36475458, 6.21549828]])
+        assert np.allclose(val, [[1.6136458, 7.00036006], [12.09938081, 7.56874402]])
 
     @pytest.mark.mechanism
     @pytest.mark.transfer_mechanism
@@ -1855,7 +1855,6 @@ class TestIntegratorMode:
         T.parameters.integrator_mode.set(True, context=C)
 
         assert T.parameters.integrator_mode.get(C) is True
-        assert T.has_integrated is True
         assert T.integrator_function is integrator_function
 
         C.run({T: [[1.0], [1.0], [1.0]]})
