@@ -193,6 +193,31 @@ class TestScheduler:
                             [np.array([[2.]]), np.array([[1.]])]]
         assert np.allclose(expected_results, np.asfarray(C.results))
 
+    def test_default_condition_1(self):
+        A = pnl.TransferMechanism(name='A')
+        B = pnl.TransferMechanism(name='B')
+        C = pnl.TransferMechanism(name='C')
+
+        comp = pnl.Composition(pathways=[[A, C], [A, B, C]])
+        comp.scheduler.add_condition(A, AtPass(1))
+        comp.scheduler.add_condition(B, Always())
+
+        output = list(comp.scheduler.run())
+        expected_output = [B, A, B, C]
+        assert output == pytest.helpers.setify_expected_output(expected_output)
+
+    def test_default_condition_2(self):
+        A = pnl.TransferMechanism(name='A')
+        B = pnl.TransferMechanism(name='B')
+        C = pnl.TransferMechanism(name='C')
+
+        comp = pnl.Composition(pathways=[[A, B], [C]])
+        comp.scheduler.add_condition(C, AtPass(1))
+
+        output = list(comp.scheduler.run())
+        expected_output = [A, B, {C, A}]
+        assert output == pytest.helpers.setify_expected_output(expected_output)
+
 
 class TestLinear:
 
@@ -1514,3 +1539,109 @@ class TestFeedback:
             assert counter.execution_count == 1 if in_one_pass else 3
             assert response.execution_count == 1
         assert np.allclose(result, expected)
+
+    @pytest.mark.composition
+    @pytest.mark.parametrize("mode", ['Python',
+                                      # 'LLVM' mode is not supported, because synchronization of compiler and python values during execution is not implemented.
+                                      pytest.param('LLVMExec', marks=pytest.mark.llvm),
+                                      pytest.param('LLVMRun', marks=pytest.mark.llvm),
+                                      pytest.param('PTXExec', marks=[pytest.mark.llvm, pytest.mark.cuda]),
+                                      pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
+                                     ])
+    @pytest.mark.parametrize("condition,scale,expected_result",
+                             [(pnl.BeforeNCalls, TimeScale.TRIAL, [[.05, .05]]),
+                              (pnl.BeforeNCalls, TimeScale.PASS, [[.05, .05]]),
+                              (pnl.EveryNCalls, None, [[0.05, .05]]),
+                              (pnl.AtNCalls, TimeScale.TRIAL, [[.25, .25]]),
+                              (pnl.AtNCalls, TimeScale.RUN, [[.25, .25]]),
+                              (pnl.AfterNCalls, TimeScale.TRIAL, [[.25, .25]]),
+                              (pnl.AfterNCalls, TimeScale.PASS, [[.05, .05]]),
+                              (pnl.WhenFinished, None, [[1.0, 1.0]]),
+                              (pnl.WhenFinishedAny, None, [[1.0, 1.0]]),
+                              (pnl.WhenFinishedAll, None, [[1.0, 1.0]]),
+                              (pnl.All, None, [[1.0, 1.0]]),
+                              (pnl.Any, None, [[1.0, 1.0]]),
+                              (pnl.Not, None, [[.05, .05]]),
+                              (pnl.AllHaveRun, None, [[.05, .05]]),
+                              (pnl.Always, None, [[0.05, 0.05]]),
+                              (pnl.AtPass, None, [[.3, .3]]),
+                              (pnl.AtTrial, None, [[0.05, 0.05]]),
+                              #(pnl.Never), #TODO: Find a good test case for this!
+                            ])
+    def test_scheduler_conditions(self, mode, condition, scale, expected_result):
+        decisionMaker = pnl.DDM(
+                        function=pnl.DriftDiffusionIntegrator(starting_point=0,
+                                                              threshold=1,
+                                                              noise=0.0),
+                        reset_stateful_function_when=pnl.AtTrialStart(),
+                        output_ports=[pnl.DECISION_VARIABLE, pnl.RESPONSE_TIME],
+                        name='DDM')
+
+        response = pnl.ProcessingMechanism(size=2, name="GATE")
+
+        comp = pnl.Composition()
+        comp.add_linear_processing_pathway([decisionMaker, response])
+
+        if condition is pnl.BeforeNCalls:
+            comp.scheduler.add_condition(response, condition(decisionMaker, 5,
+                                                             time_scale=scale))
+        elif condition is pnl.AtNCalls:
+            comp.scheduler.add_condition(response, condition(decisionMaker, 5,
+                                                             time_scale=scale))
+        elif condition is pnl.AfterNCalls:
+            # Mechanisms run only once per PASS unless they are in
+            # 'run_until_finished' mode.
+            c = 1 if scale is TimeScale.PASS else 5
+            comp.scheduler.add_condition(response, condition(decisionMaker, c,
+                                                             time_scale=scale))
+        elif condition is pnl.EveryNCalls:
+            comp.scheduler.add_condition(response, condition(decisionMaker, 1))
+        elif condition is pnl.WhenFinished:
+            comp.scheduler.add_condition(response, condition(decisionMaker))
+        elif condition is pnl.WhenFinishedAny:
+            comp.scheduler.add_condition(response, condition(decisionMaker))
+        elif condition is pnl.WhenFinishedAll:
+            comp.scheduler.add_condition(response, condition(decisionMaker))
+        elif condition is pnl.All:
+            comp.scheduler.add_condition(response, condition(pnl.WhenFinished(decisionMaker)))
+        elif condition is pnl.Any:
+            comp.scheduler.add_condition(response, condition(pnl.WhenFinished(decisionMaker)))
+        elif condition is pnl.Not:
+            comp.scheduler.add_condition(response, condition(pnl.WhenFinished(decisionMaker)))
+        elif condition is pnl.AllHaveRun:
+            comp.scheduler.add_condition(response, condition(decisionMaker))
+        elif condition is pnl.Always:
+            comp.scheduler.add_condition(response, condition())
+        elif condition is pnl.AtPass:
+            comp.scheduler.add_condition(response, condition(5))
+        elif condition is pnl.AtTrial:
+            comp.scheduler.add_condition(response, condition(0))
+
+        result = comp.run([0.05], bin_execute=mode)
+        #HACK: The result is an object dtype in Python mode for some reason?
+        if mode == 'Python':
+            result = np.asfarray(result[0])
+        assert np.allclose(result, expected_result)
+
+
+    @pytest.mark.composition
+    @pytest.mark.parametrize("mode", ['Python',
+                                      pytest.param('LLVMRun', marks=pytest.mark.llvm),
+                                      pytest.param('PTXRun', marks=[pytest.mark.llvm, pytest.mark.cuda]),
+                                     ])
+    @pytest.mark.parametrize("condition,scale,expected_result",
+                             [(pnl.AtTrial, None, [[[1.0]], [[2.0]]]),
+                             ])
+    def test_run_term_conditions(self, mode, condition, scale, expected_result):
+        incrementing_mechanism = pnl.ProcessingMechanism(
+            function=pnl.SimpleIntegrator
+        )
+        comp = pnl.Composition(
+            pathways=[incrementing_mechanism]
+        )
+        comp.scheduler.termination_conds = {
+            pnl.TimeScale.RUN: condition(2)
+        }
+        r = comp.run(inputs=[1], num_trials=5, bin_execute=mode)
+        assert np.allclose(r, expected_result[-1])
+        assert np.allclose(comp.results, expected_result)
